#!/opt/nimbus/bin/perl
#################################################################
# CodeWizard:  perl 
# This code was generated with the Nimsoft CodeWizard version 1.70
# Date: Friday, February 01, 2013

#define VERSION 1.0

use strict;
use POSIX;
use File::Basename;
use Switch;
use Win32::TieRegistry(Delimiter => "/");
use Data::Dumper;
my $base_name;
my $nim_root;
my $perl_lib;
my $probe_name;

BEGIN
{
  $base_name = basename($0);

  my $path = $Registry->{"HKEY_LOCAL_MACHINE/SOFTWARE/Wow6432Node/Nimbus Software/NimBUS Installation//Home"};
  my @dir = split(/\\/, $path);
  $nim_root = join('/', @dir);
  push(@dir, 'perllib');
  $perl_lib = join('/', @dir);
}

#use hardcoded lib path for standalone executable?
#use lib $perl_lib;
use lib 'C:/Program Files (x86)/Nimsoft/perllib';

use Nimbus::API;
use Nimbus::Session;
use Nimbus::CFG;
use Nimbus::PDS;

my $LOG;
my $timeout = 30;

sub doLog
{
  my $message = shift;
  print $LOG "$message\n";
}

sub doExit
{
  my $message = shift;

  print("$message\n\n");
  print "Press <Enter> to exit:";
  <STDIN>;
  doLog($message);
  close($LOG);
  exit(1);
}

my $doNimRequestErr;

sub doNimRequest
{
  my $ip = shift;
  my $port = shift;
  my $command = shift;
  my $pds_in = shift; 
  my $timeout = shift;

  my $pds_out = undef;
  for (my $itry = 1; $itry <= 3; ++$itry)
  {
    my ($rc, $pds_data) = nimRequest($ip, $port, $command,
                                       $pds_in->data(), $timeout);
    if ($rc)
    {
      my $err = nimError2Txt($rc);
      $doNimRequestErr = "rc=$rc err=$err from doNimRequestErr" .
        " ip=$ip port=$port command=$command on try $itry";
      doLog($doNimRequestErr);
    }
    else
    {
      $doNimRequestErr = undef;
      $pds_out = Nimbus::PDS->new($pds_data);
      last;
    }
  }
  return($pds_out);
}

my $conf_name = basename($0, ('.pl', '.exe', '.bin'));
my $temp = $ENV{'TEMP'} // $ENV{'TMP'};
my $log_file;
if (defined($temp))
{
  $log_file = "$temp/$conf_name.log";
}
else
{
  $log_file = "$conf_name.log";
}
print("log_file=$log_file\n");
open $LOG, '>', $log_file;
doLog("perl_lib=$perl_lib");
doLog("conf_name=$conf_name");
doLog("temp=$temp");

foreach my $key(keys(%ENV))
{
  my $env = $ENV{$key};
  doLog("ENV{$key}=$ENV{$key}");
}

my $hub_ip;
my $probe_name;
my $log_level;
my $log_file;
my $sid;
my $do_login = 0;
my $package;
my $package_version;
my $section;
my $file;

my $probe_address = $ARGV[$0];
doLog("probe_address=$probe_address");
for (my $i = 1; $i < @ARGV; $i++)
{
  doLog("ARGV[$i]=$ARGV[$i]");
  my $arg = $ARGV[$i];
  my $flag = substr($arg, 0, 2);
  switch($flag)
  {
    case '-c' {$hub_ip = substr($arg, 2)}
    case '-n' {$probe_name = substr($arg, 2)}
    case '-d' {$log_level = substr($arg, 2)}
    case '-l' {$log_file = substr($arg, 2)}
    case '-s' {$sid = substr($arg, 2)}
    case '-L' {$do_login = 1}
    case '-P' {$package = substr($arg, 2)}
    case '-V' {$package_version = substr($arg, 2)}
    case '-S' {$section = substr($arg, 2)}
    case '-F' {$file = substr($arg, 2)}
  }
}
doLog("hub_ip=$hub_ip");
doLog("probe_name=$probe_name");
doLog("log_level=$log_level");
doLog("log_file=$log_file");
doLog("sid=$sid");
doLog("do_login=$do_login");
doLog("package=$package");
doLog("package_version=$package_version");
doLog("section=$section");

my $hub_port = 48002;

if (!defined($sid))
{
  #my lab environment
  $probe_address = '/centos65int123dom/centos65int123hub/w7entsp1int121/forecast';
  $probe_name = 'forecast';
  $sid = nimLogin('administrator', 'P@ssW0rd');
  print("sid=$sid\n");
}

doExit("not logged in to Nimbus!") if (!defined($sid) or ($sid eq ''));

my $rc = nimChangeLogin($sid);
if ($rc != NIME_OK)
{
  my $err = nimError2Txt($rc);
  doExit("rc=$rc err=$err from nimChangeLogin");
}

my @dir = split('/', $probe_address);
pop(@dir);
push(@dir, 'controller');
my $controller_address = join('/', @dir);
doLog("controller_address=$controller_address");

my $pds_in = Nimbus::PDS->new();
$pds_in->string('name', $controller_address);
my $pds_out = doNimRequest($hub_ip, $hub_port, 'nametoip', $pds_in, $timeout);
doExit("error $doNimRequestErr nametoip") if (!defined($pds_out));
my $pds_hash = $pds_out->asHash();
my $ip = $pds_hash->{ip};
doExit("nimRequest nametoip should have returned an ip") if (!defined($ip));
my $port = $pds_hash->{port};
doExit("nimRequest nametoip should have returned a port") if (!defined($ip));

doLog("ip=$ip");
doLog("port=$port");
print("controller_address=$controller_address ip=$ip port=$port\n");

$pds_in->reset();
$pds_in->string('name', $probe_name);
my $pds_out = doNimRequest($ip, $port, 'probe_list', $pds_in, $timeout);
doExit("error $doNimRequestErr quering probe configuration and working directory") if (!defined($pds_out));

my $pds_hash = $pds_out->asHash();
my $workdir = $pds_hash->{$probe_name}->{workdir};
doExit("nimRequest probe_list should have returned a workdir") if (!defined($workdir));

my $config = $pds_hash->{$probe_name}->{config};
doExit("nimRequest probe_list should have returned a config") if (!defined($config));
print("workdir=$workdir config=$config\n");

$pds_in->reset();
$pds_in->string('directory', $workdir);
$pds_in->string('file', $config);

$pds_out = doNimRequest($ip, $port, 'text_file_get', $pds_in,
  $timeout);
doExit("error $doNimRequestErr getting probe configuration file") if (!defined($pds_out));

my $pds_pch = $pds_out->get('file_content', PDS_PCH);
doExit("nimRequest text_file_get should have returned a PDS_PCH") if (! $pds_pch);

#print(Dumper($pds_pch));
#check if configuration already contains an editing history
if (!($pds_pch =~ /<history>/))
{
  $pds_pch .= "<history>\n</history>";
}

my $version = 0;
while($pds_pch =~ /version$version/) {$version++};
my $config_version = "$config.$version";

open my $CFG, '>', "$config_version";
print $CFG $pds_pch;
close($CFG);

#now call notepad
system("notepad $config_version");

open $CFG, '<', "$config_version";
my @file_contents = <$CFG>;
close($CFG);
my $file_contents = join('', @file_contents);

$pds_in->reset();
$pds_in->string('directory', $workdir);
$pds_in->string('file', "$config_version");
$pds_in->string('file_contents', $file_contents);

$pds_out = doNimRequest($ip, $port, 'text_file_put', $pds_in,
  $timeout);
doExit("error $doNimRequestErr putting configuration file $config_version") if (!defined($pds_out));

$pds_in->reset();
$pds_in->string('directory', $workdir);
$pds_in->string('file', "$config");
$pds_in->string('file_contents', $file_contents);

$pds_out = doNimRequest($ip, $port, 'text_file_put', $pds_in,
  $timeout);
doExit("error $doNimRequestErr putting configuration file $config") if (!defined($pds_out));

#now update the editing history

my $username = $ENV{'USERNAME'};
my $now = time();
my $stamp = strftime('%Y-%m-%d %H:%M:%S', localtime($now));

print "\nEnter a description or just <Enter>: ";
my $description = <STDIN>;
chomp($description);
$description = 'No description given' if ($description eq '');

$pds_in->reset();
$pds_in->string('name', $probe_name);
$pds_in->string('section', 'history');
$pds_in->string('key', "version$version");
$pds_in->string('value', "Edited by $username at $stamp: $description");
doLog("username=$username stamp=$stamp");

$pds_out = doNimRequest($ip, $port, 'probe_config_set', $pds_in,
  $timeout);
doExit("error $doNimRequestErr updating editing history") if (!defined($pds_out));

close($LOG);
